﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Media;
using Telerik.Windows.Controls;

namespace WBTest
{
    public class MyCustomShape : RadDiagramShape
    {
        private ObservableCollection<Point> vertexPoints = new ObservableCollection<Point>();
        private List<Point> relativePoints = new List<Point>();

        public MyCustomShape()
        {
            this.vertexPoints.Add(new Point(0, 0));
            this.vertexPoints.Add(new Point(-50, 50));
            this.vertexPoints.Add(new Point(75, 50));
            this.vertexPoints.Add(new Point(75, 75));

            PathGeometry g = new PathGeometry();
            PathFigure f = new PathFigure();
            var firstVertexPoint = this.vertexPoints.FirstOrDefault();
            var startPoint = new Point(firstVertexPoint.X, firstVertexPoint.Y);

            f.StartPoint = startPoint;
            f.IsClosed = true;

            foreach (var point in this.VertexPoints.Skip(1))
            {
                LineSegment s = new LineSegment();
                s.Point = new Point(point.X, point.Y);
                f.Segments.Add(s);
            }

            g.Figures.Add(f);

            this.Geometry = g;
            this.Position = new Point(100, 100);

            this.CalculateRelativePointsFromVertices();
        }

        private void CalculateRelativePointsFromVertices()
        {
            var minPoint = new Point(this.vertexPoints.Min(p => p.X), this.vertexPoints.Min(p => p.Y));
            var startWidth = this.vertexPoints.Max(p => p.X) - minPoint.X;
            var startHeight = this.vertexPoints.Max(p => p.Y) - minPoint.Y;
            if (startWidth <= 0 || startHeight <= 0)
            {
                // Invalid vetices ? 
            }
            else
            {
                foreach (var point in this.VertexPoints)
                {
                    this.relativePoints.Add(new Point(Math.Round(point.X / startWidth, 4), Math.Round(point.Y / startHeight, 4)));
                }
            }
        }

        public ObservableCollection<Point> VertexPoints
        {
            get
            {
                return this.vertexPoints;
            }
        }

        public void UpdateVertices()
        {
            this.vertexPoints.Clear();
            foreach (var relativePoint in this.relativePoints)
            {
                this.vertexPoints.Add(new Point(relativePoint.X * this.ActualWidth, relativePoint.Y * this.ActualHeight));
            }
        }
    }
}
